@@ -1,26 +1,16 @@
-# $Author: ddumont $
-# $Date: 2009-04-07 13:16:38 +0200 (Tue, 07 Apr 2009) $
-# $Revision: 920 $
-
-#    Copyright (c) 2008 Dominique Dumont.
-#
-#    This file is part of Config-Model-TkUI.
-#
-#    Config-Model is free software; you can redistribute it and/or
-#    modify it under the terms of the GNU Lesser Public License as
-#    published by the Free Software Foundation; either version 2.1 of
-#    the License, or (at your option) any later version.
-#
-#    Config-Model is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#    Lesser Public License for more details.
-#
-#    You should have received a copy of the GNU Lesser Public License
-#    along with Config-Model; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-
+# 
+# This file is part of Config-Model-TkUI
+# 
+# This software is Copyright (c) 2010 by Dominique Dumont.
+# 
+# This is free software, licensed under:
+# 
+#   The GNU Lesser General Public License, Version 2.1, February 1999
+# 
 package Config::Model::Tk::HashEditor ;
+BEGIN {
+  $Config::Model::Tk::HashEditor::VERSION = '1.310';
+}
 
 use strict;
 use warnings ;
@@ -28,13 +18,13 @@ use Carp ;
 use Log::Log4perl ;
 
 use base qw/Config::Model::Tk::HashViewer/;
-use vars qw/$VERSION $icon_path/ ;
+use vars qw/$icon_path/ ;
 use subs qw/menu_struct/ ;
 use Tk::Dialog ;
 use Tk::Photo ;
 use Tk::Balloon ;
-
-$VERSION = sprintf "1.%04d", q$Revision: 920 $ =~ /(\d+)/;
+use Tk::ErrorDialog ;
+use Config::Model::Tk::NoteEditor ;
 
 Construct Tk::Widget 'ConfigModelHashEditor';
 
@@ -58,130 +48,206 @@ sub ClassInit {
     # cw->Advertise(name=>$widget);
 }
 
+# list of widget that must be activated (0 in this table) when 
+# item is selected in listbox or entry is not empty
+
+my %widget_activation_table = ( add  => { tklist => 1, entry => 0 },
+				mv   => { tklist => 0, entry => 0 },
+				cp   => { tklist => 0, entry => 0 },
+				up   => { tklist => 0, entry => 1 },
+				down => { tklist => 0, entry => 1 },
+				del  => { tklist => 0, entry => 1 },
+			      );
+
+
 sub Populate { 
     my ($cw, $args) = @_;
     my $hash = $cw->{hash} = delete $args->{-item} 
       || die "HashEditor: no -item, got ",keys %$args;
     delete $args->{-path} ;
+    $cw->{store_cb} = delete $args->{-store_cb} || die __PACKAGE__,"no -store_cb" ;
 
     unless (defined $up_img) {
 	$up_img   = $cw->Photo(-file => $icon_path.'up.png');
 	$down_img = $cw->Photo(-file => $icon_path.'down.png');
     }
 
-    $cw->add_header(Edit => $hash) ;
+    $cw->add_header(Edit => $hash)->pack(@fx, -anchor => 'n') ;
 
     my $inst = $hash->instance ;
 
-    my $elt_button_frame = $cw->Frame->pack(@fbe1) ;
+    # frame for element list
+    my $elt_frame = $cw->Frame(qw/-relief raised -borderwidth 2/)
+      ->pack(@fbe1,-side => 'left', -anchor => 'w') ;
 
-    my $elt_frame = $elt_button_frame->Frame(qw/-relief raised -borderwidth 2/)
-                                     ->pack(@fbe1,-side => 'left') ;
-    $elt_frame -> Label(-text => $hash->element_name.' elements') -> pack() ;
+    $elt_frame -> Label(-text => $hash->element_name.' elements')
+      -> pack(@fx) ;
 
+    # element list
     my $tklist = $elt_frame ->Scrolled ( 'Listbox',
 					 -selectmode => 'single',
 					 -scrollbars => 'oe',
 					 -height => 6,
-				       )
-                            -> pack(@fbe1, -side => 'left') ;
-
+				       );
+    $tklist-> pack(@fbe1, -side => 'left') ;
     $tklist->insert( end => $hash->get_all_indexes) ;
+    $cw->Advertise( tklist => $tklist) ;
 
-    my $right_frame = $elt_button_frame->Frame->pack(@fxe1, -side => 'left');
-
-    $cw->add_info($cw) ;
-    $cw->add_summary_and_description($hash) ;
-
-    my $item_frame = $right_frame->Frame(-relief => 'groove',-bd => 4 )
-      ->pack( @fxe1);
+    my $item_frame = $cw->Frame(qw/-borderwidth 1 -relief groove/)
+      ->pack( @fx, -anchor => 'n');
 
     my $balloon = $cw->Balloon(-state => 'balloon') ;
 
+    my $label_keep_frame = $item_frame->Frame->pack(@fxe1) ;
     my $item = '';
     my $keep = 0 ;
-    my $label_frame = $item_frame->Frame->pack( @fxe1);
-    $label_frame -> Label (-text => 'Item:')->pack(@fxe1,-side => 'left') ;
-    my $keep_b = $label_frame -> Checkbutton (-variable => \$keep, 
+    $label_keep_frame -> Label (-text => 'Item:')
+      -> pack(-side => 'left', -anchor => 'w') ;
+
+    my $keep_b = $label_keep_frame -> Checkbutton (-variable => \$keep, 
 					    -text => 'keep')
-      -> pack  (-side => 'left') ;
+      -> pack  (qw/-side right -anchor e/) ;
     $balloon->attach($keep_b, 
 		     -msg => 'keep entry in widget after add, move or copy');
 
-    my $entry = $item_frame -> Entry (-textvariable => \$item )
-      -> pack  (@fxe1) ;
+    # Entry
+    my $entry = $item_frame -> Entry (-textvariable => \$item);
+    $entry -> pack  (@fxe1, qw/-side top -anchor n/) ;
     $balloon -> attach($entry, 
 		       -msg => 'enter item name to add, copy to, or move to') ;
+    $cw->Advertise(entry => $entry );
 
-    my $button_frame = $item_frame->Frame->pack( );
+    # bind both entries to update correctly the state of all buttons
+    my $bound_sub = sub { 
+	$cw->update_state(entry => $item , tklist => $tklist->curselection || 0) 
+    };
+    $entry -> bind( '<KeyPress>'       , $bound_sub );
+    $tklist-> bind( '<<ListboxSelect>>', $bound_sub );
 
+    # frame for all buttons
+    my $button_frame = $item_frame->Frame->pack(@fxe1,qw/-anchor n/ );
+
+    # add button
     my $addb = $button_frame 
       -> Button(-text => "Add",
 		-command => sub { $cw->add_entry($item); 
 				  $item = '' unless $keep;
+				  &$bound_sub ;
 			      },
 		-anchor => 'e',
-	       )->pack(-side => 'left');
+	       );
+    $addb -> pack(@fxe1, qw/-side left/);
+    $cw->Advertise( add    => $addb) ;
     my $add_str = $hash->ordered ? " after selection" : '' ;
     $balloon->attach($addb,
-		     -msg => "add entry".$add_str);
+		     -msg => "fill field above and click to add new entry".$add_str);
 
+    # copy button
     my $cp_b = $button_frame 
       -> Button(-text => 'Copy',
 		-command => sub { $cw->copy_selected_in($item); 
-				  $item = '' unless $keep;},
+				  $item = '' unless $keep;
+				  &$bound_sub ;
+			      },
 		-anchor => 'e',
-	       )
-	-> pack(-side => 'left');
-    $balloon->attach($cp_b,
-		     -msg => "copy selected item in entry");
+	       );
+    $cp_b -> pack(qw/-side right/);
+    $cw->Advertise(@fxe1, cp     => $cp_b) ;
+    $balloon->attach($cp_b, -msg => "copy selected item in entry");
 
-
-    my $mv_b = $button_frame 
-      -> Button(-text => 'Move',
+    # rename button
+    my $rename_b = $button_frame 
+      -> Button(-text => 'Rename',
 		-command => sub { $cw->move_selected_to($item); 
-				  $item = '' unless $keep; },
+				  $item = '' unless $keep;
+				  &$bound_sub,
+			      },
 		-anchor => 'e',
-	       )
-	-> pack(-side => 'left');
-    $balloon->attach($mv_b,
-		     -msg => "move selected item in entry");
+	       );
+    $rename_b -> pack(@fxe1, -side => 'left');
+    $cw->Advertise( mv     => $rename_b) ;
+    $balloon->attach($rename_b,
+		     -msg => "rename selected item in entry");
 
     if ($hash->ordered) {
-	my $mv_up_down_frame = $right_frame->Frame->pack( @fxe1);
-	$mv_up_down_frame->Button(-image => $up_img,
-				  -command => sub { $cw->move_selected_up ;} ,
-				 )-> pack( -side =>'left' , @fxe1);
+	my $mv_up_down_frame = $item_frame->Frame->pack( @fx, qw/-anchor n/);
+	my $up_b = $mv_up_down_frame->Button(-image => $up_img,
+					     -command => sub { $cw->move_selected_up ;} ,
+					    );
 
-	$mv_up_down_frame->Button(-image => $down_img,
-				  -command => sub { $cw->move_selected_down ;} ,
-				 )-> pack( -side =>'left' , @fxe1);
+	my $down_b = $mv_up_down_frame->Button(-image => $down_img,
+					       -command => sub { $cw->move_selected_down ;} ,
+					      );
+	$up_b -> pack( -side =>'left' , @fxe1);
+	$down_b -> pack( -side =>'left' , @fxe1);
+
+	$cw->Advertise( up => $up_b ) ;
+	$cw->Advertise( down => $down_b ) ;
     }
 
-    my $del_rm_frame =  $right_frame->Frame->pack( @fxe1);
+    my $del_rm_frame =  $item_frame->Frame->pack( @fx, qw/-side top -anchor n/);
 
-    $del_rm_frame->Button(-text => 'Delete selected',
-			  -command => sub { $cw->delete_selection; 
-					    $item = '' unless $keep;} ,
-			 )-> pack( -side =>'left' , @fxe1);
+    my $del_b = $del_rm_frame->Button(-text => 'Remove selected',
+				      -command => sub { $cw->delete_selection; 
+							$item = '' unless $keep;
+							&$bound_sub;
+						    } ,
+				     ) ;
+    $del_b -> pack( -side => 'left' , @fxe1);
+    $cw->Advertise( del => $del_b ) ;
 
     $del_rm_frame -> Button ( -text => 'Remove all',
-			      -command => sub { $hash->clear ; 
-						$tklist->delete(0,'end');
-						$cw->reload_tree;
-						$item = '';
-					    },
+			      -command => sub {	$cw->remove_all_elements; $item = '';},
 			    ) -> pack(-side => 'left', @fxe1) ;
 
-    $cw->{tklist} = $tklist ;
+    $cw->ConfigModelNoteEditor( -object => $hash )->pack( qw/-anchor n/);
+
+    # set all buttons to their default state
+    $cw->update_state(tklist => '', entry => '') ;
+
+    $cw->add_info_button()->pack( @fx,qw/-anchor n/) ;
+    $cw->add_summary($hash)->pack(@fx) ;
+    $cw->add_description($hash)->pack(@fx) ;
 
     $cw->Tk::Frame::Populate($args) ;
 }
 
+sub remove_all_elements {
+    my $cw = shift;
+    my $dialog = $cw->Dialog(-title => "Delete ?",
+	-text => "Are you sure you want to delete all elements ?",
+	-buttons => [qw/Yes No/],
+	-default_button => 'Yes',
+	) ;
+    my $answer = $dialog->Show ;
+    return unless $answer eq 'Yes';
+    $cw->{hash}->clear ; 
+    $cw->Subwidget('tklist')->delete(0,'end');
+    $cw->reload_tree(1);
+}
+
+# update buttons state according to entry and list widget
+# this method is called whenever one of them changes its content
+sub update_state {
+    my ($cw,%content) = @_ ;
+
+    my $wat = \%widget_activation_table ;
+
+    foreach my $button (keys %$wat) {
+	my $new = 1 ;
+	foreach my $c (keys %content) {
+	    $new &&= $wat->{$button}{$c} || $content{$c} ;
+	}
+	my $subwidget = $cw->Subwidget($button) || next;
+	$subwidget->configure( -state => $new ? 'normal':'disabled') ;
+    }
+}
+
+
 sub add_entry {
     my $cw = shift;
     my $add = shift;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my $hash = $cw->{hash};
 
     $logger->debug("add_entry: $add");
@@ -242,7 +308,7 @@ sub add_and_sort_item {
     my $cw  = shift;
     my $add = shift ;
 
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my $idx = 0;
     my $added = 0 ;
 
@@ -270,7 +336,7 @@ sub add_item {
     my $add = shift ;
 
     my $hash = $cw->{hash};
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
 
     # add entry in tklist
     if ($hash->ordered) {
@@ -287,7 +353,7 @@ sub add_item {
 sub get_selection {
     my $cw =shift;
     my $what = shift ;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my @from_idx = $tklist->curselection() ;
     if (not @from_idx) {
 	$cw->Dialog(-title => "$what selection error",
@@ -301,7 +367,7 @@ sub get_selection {
 sub copy_selected_in {
     my $cw =shift;
     my $to_name = shift ;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my @from_idx = $cw->get_selection('copy') or return 0 ;
     my $from_name = $tklist->get(@from_idx);
 
@@ -329,7 +395,7 @@ sub copy_selected_in {
 sub move_selected_to {
     my $cw =shift;
     my $to_name = shift ;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my @from_idx = $cw->get_selection('move') or return 0 ;
     my $from_name = $tklist->get(@from_idx);
 
@@ -365,7 +431,7 @@ sub move_selected_to {
 
 sub move_selected_up {
     my $cw =shift;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my @idx = $tklist->curselection() ;
 
     return unless @idx and $idx[0] > 0;
@@ -388,7 +454,7 @@ sub move_selected_up {
 
 sub move_selected_down {
     my $cw =shift;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my @idx = $tklist->curselection() ;
     my $hash = $cw->{hash};
     my @h_idx =  $hash->get_all_indexes ;
@@ -412,7 +478,7 @@ sub move_selected_down {
 
 sub delete_selection {
     my $cw = shift ;
-    my $tklist = $cw->{tklist} ;
+    my $tklist = $cw->Subwidget('tklist') ;
     my $hash = $cw->{hash};
 
     foreach ($tklist->curselection()) {
@@ -443,7 +509,7 @@ sub store {
 
 sub reload_tree {
     my $cw = shift ;
-    $cw->parent->parent->parent->parent->reload(1) ;
+    $cw->{store_cb}->() ;
 }
 
 
